.TH PTHREAD_SIGNAL 3 LinuxThreads

.XREF pthread_kill
.XREF sigwait

.SH NAME
pthread_sigmask, pthread_kill, sigwait \- handling of signals in threads

.SH SYNOPSIS
#include <pthread.h>
.br
#include <signal.h>

int pthread_sigmask(int how, const sigset_t *newmask, sigset_t *oldmask);

int pthread_kill(pthread_t thread, int signo);

int sigwait(const sigset_t *set, int *sig);

.SH DESCRIPTION

\fBpthread_sigmask\fP changes the signal mask for the calling thread as
described by the \fIhow\fP and \fInewmask\fP arguments. If \fIoldmask\fP is not
\fBNULL\fP, the previous signal mask is stored in the location pointed to
by \fIoldmask\fP. 

The meaning of the \fIhow\fP and \fInewmask\fP arguments is the same as for
\fBsigprocmask\fP(2). If \fIhow\fP is \fBSIG_SETMASK\fP, the signal mask is set to
\fInewmask\fP. If \fIhow\fP is \fBSIG_BLOCK\fP, the signals specified to \fInewmask\fP
are added to the current signal mask.  If \fIhow\fP is \fBSIG_UNBLOCK\fP, the
signals specified to \fInewmask\fP are removed from the current signal
mask.

Recall that signal masks are set on a per-thread basis, but signal
actions and signal handlers, as set with \fBsigaction\fP(2), are shared
between all threads.

\fBpthread_kill\fP send signal number \fIsigno\fP to the thread
\fIthread\fP. The signal is delivered and handled as described in
\fBkill\fP(2).

\fBsigwait\fP suspends the calling thread until one of the signals
in \fIset\fP is delivered to the calling thread. It then stores the number
of the signal received in the location pointed to by \fIsig\fP and
returns. The signals in \fIset\fP must be blocked and not ignored on
entrance to \fBsigwait\fP. If the delivered signal has a signal handler
function attached, that function is \fInot\fP called.

.SH CANCELLATION

\fBsigwait\fP is a cancellation point.

.SH "RETURN VALUE"

On success, 0 is returned. On failure, a non-zero error code is returned.

.SH ERRORS

The \fBpthread_sigmask\fP function returns the following error codes
on error:
.RS
.TP
\fBEINVAL\fP
\fIhow\fP is not one of \fBSIG_SETMASK\fP, \fBSIG_BLOCK\fP, or \fBSIG_UNBLOCK\fP

.TP
\fBEFAULT\fP
\fInewmask\fP or \fIoldmask\fP point to invalid addresses
.RE

The \fBpthread_kill\fP function returns the following error codes
on error:
.RS
.TP
\fBEINVAL\fP
\fIsigno\fP is not a valid signal number

.TP
\fBESRCH\fP
the thread \fIthread\fP does not exist (e.g. it has already terminated)
.RE

The \fBsigwait\fP function never returns an error.

.SH AUTHOR
Xavier Leroy <Xavier.Leroy@inria.fr>

.SH "SEE ALSO"
\fBsigprocmask\fP(2),
\fBkill\fP(2),
\fBsigaction\fP(2),
\fBsigsuspend\fP(2).

.SH NOTES

For \fBsigwait\fP to work reliably, the signals being waited for must be
blocked in all threads, not only in the calling thread, since
otherwise the POSIX semantics for signal delivery do not guarantee
that it's the thread doing the \fBsigwait\fP that will receive the signal.
The best way to achieve this is block those signals before any threads
are created, and never unblock them in the program other than by
calling \fBsigwait\fP.

.SH BUGS

Signal handling in LinuxThreads departs significantly from the POSIX
standard. According to the standard, ``asynchronous'' (external)
signals are addressed to the whole process (the collection of all
threads), which then delivers them to one particular thread. The
thread that actually receives the signal is any thread that does
not currently block the signal.

In LinuxThreads, each thread is actually a kernel process with its own
PID, so external signals are always directed to one particular thread.
If, for instance, another thread is blocked in \fBsigwait\fP on that
signal, it will not be restarted.

The LinuxThreads implementation of \fBsigwait\fP installs dummy signal
handlers for the signals in \fIset\fP for the duration of the wait. Since
signal handlers are shared between all threads, other threads must not
attach their own signal handlers to these signals, or alternatively
they should all block these signals (which is recommended anyway --
see the Notes section).
